# -*- coding: utf-8 -*-
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

from ansible_collections.community.internal_test_tools.tests.unit.utils.fetch_url_module_framework import (
    BaseTestModule,
    FetchUrlCall,
)

from ansible_collections.community.dns.plugins.modules import hosttech_dns_zone_info

# These imports are needed so patching below works
import ansible_collections.community.dns.plugins.module_utils.http  # noqa

from .hosttech import (
    expect_wsdl_authentication,
    expect_wsdl_value,
    validate_wsdl_call,
    HOSTTECH_WSDL_DEFAULT_ZONE_RESULT,
    HOSTTECH_WSDL_ZONE_NOT_FOUND,
    HOSTTECH_JSON_ZONE_GET_RESULT,
    HOSTTECH_JSON_ZONE_2_GET_RESULT,
    HOSTTECH_JSON_ZONE_LIST_RESULT,
)

try:
    import lxml.etree
    HAS_LXML_ETREE = True
except ImportError:
    HAS_LXML_ETREE = False


@pytest.mark.skipif(not HAS_LXML_ETREE, reason="Need lxml.etree for WSDL tests")
class TestHosttechDNSZoneInfoWSDL(BaseTestModule):
    MOCK_ANSIBLE_MODULEUTILS_BASIC_ANSIBLEMODULE = 'ansible_collections.community.dns.plugins.modules.hosttech_dns_zone_info.AnsibleModule'
    MOCK_ANSIBLE_MODULEUTILS_URLS_FETCH_URL = 'ansible_collections.community.dns.plugins.module_utils.http.fetch_url'

    def test_unknown_zone(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_username': 'foo',
            'hosttech_password': 'bar',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('POST', 200)
            .expect_content_predicate(validate_wsdl_call([
                expect_wsdl_authentication('foo', 'bar'),
                expect_wsdl_value(
                    [lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'getZone').text, 'sZoneName'],
                    'example.org',
                    ('http://www.w3.org/2001/XMLSchema', 'string')
                ),
            ]))
            .result_str(HOSTTECH_WSDL_ZONE_NOT_FOUND),
        ])

        assert result['msg'] == 'Zone not found'

    def test_unknown_zone_id(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_username': 'foo',
            'hosttech_password': 'bar',
            'zone_id': 23,
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('POST', 200)
            .expect_content_predicate(validate_wsdl_call([
                expect_wsdl_authentication('foo', 'bar'),
                expect_wsdl_value(
                    [lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'getZone').text, 'sZoneName'],
                    '23',
                    ('http://www.w3.org/2001/XMLSchema', 'string')
                ),
            ]))
            .result_str(HOSTTECH_WSDL_ZONE_NOT_FOUND),
        ])

        assert result['msg'] == 'Zone not found'

    def test_get(self, mocker):
        result = self.run_module_success(mocker, hosttech_dns_zone_info, {
            'hosttech_username': 'foo',
            'hosttech_password': 'bar',
            'zone_name': 'example.com',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('POST', 200)
            .expect_content_predicate(validate_wsdl_call([
                expect_wsdl_authentication('foo', 'bar'),
                expect_wsdl_value(
                    [lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'getZone').text, 'sZoneName'],
                    'example.com',
                    ('http://www.w3.org/2001/XMLSchema', 'string')
                ),
            ]))
            .result_str(HOSTTECH_WSDL_DEFAULT_ZONE_RESULT),
        ])

        assert result['changed'] is False
        assert result['zone_id'] == 42
        assert result['zone_name'] == 'example.com'

    def test_get_id(self, mocker):
        result = self.run_module_success(mocker, hosttech_dns_zone_info, {
            'hosttech_username': 'foo',
            'hosttech_password': 'bar',
            'zone_id': '42',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('POST', 200)
            .expect_content_predicate(validate_wsdl_call([
                expect_wsdl_authentication('foo', 'bar'),
                expect_wsdl_value(
                    [lxml.etree.QName('https://ns1.hosttech.eu/public/api', 'getZone').text, 'sZoneName'],
                    '42',
                    ('http://www.w3.org/2001/XMLSchema', 'string')
                ),
            ]))
            .result_str(HOSTTECH_WSDL_DEFAULT_ZONE_RESULT),
        ])

        assert result['changed'] is False
        assert result['zone_id'] == 42
        assert result['zone_name'] == 'example.com'
        assert result['zone_info'] == {
            'email': 'dns@hosttech.eu',
            'ttl': 10800,
        }


class TestHosttechDNSZoneInfoJSON(BaseTestModule):
    MOCK_ANSIBLE_MODULEUTILS_BASIC_ANSIBLEMODULE = 'ansible_collections.community.dns.plugins.modules.hosttech_dns_zone_info.AnsibleModule'
    MOCK_ANSIBLE_MODULEUTILS_URLS_FETCH_URL = 'ansible_collections.community.dns.plugins.module_utils.http.fetch_url'

    def test_unknown_zone(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
            .expect_query_values('query', 'example.org')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_LIST_RESULT),
        ])

        assert result['msg'] == 'Zone not found'

    def test_unknown_zone_id(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_id': 23,
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 404)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/23')
            .return_header('Content-Type', 'application/json')
            .result_json(dict(message="")),
        ])

        assert result['msg'] == 'Zone not found'

    def test_auth_error(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 401)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
            .expect_query_values('query', 'example.org')
            .result_str(''),
        ])

        assert result['msg'] == 'Cannot authenticate: Unauthorized: the authentication parameters are incorrect (HTTP status 401)'

    def test_auth_error_forbidden(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_id': 23,
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 403)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/23')
            .result_json(dict(message="")),
        ])

        assert result['msg'] == 'Cannot authenticate: Forbidden: you do not have access to this resource (HTTP status 403)'

    def test_other_error(self, mocker):
        result = self.run_module_failed(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_name': 'example.org',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 500)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
            .expect_query_values('query', 'example.org')
            .result_str(''),
        ])

        assert result['msg'].startswith('Error: GET https://api.ns1.hosttech.eu/api/user/v1/zones?')
        assert 'did not yield JSON data, but HTTP status code 500 with Content-Type' in result['msg']

    def test_get(self, mocker):
        result = self.run_module_success(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_name': 'example.com',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
            .expect_query_values('query', 'example.com')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_LIST_RESULT),
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_GET_RESULT),
        ])
        assert result['changed'] is False
        assert result['zone_id'] == 42
        assert result['zone_name'] == 'example.com'
        assert result['zone_info'] == {
            'email': 'test@example.com',
            'ttl': 10800,
            'dnssec': False,
            'dnssec_email': None,
            'ds_records': None,
        }

    def test_get_id(self, mocker):
        result = self.run_module_success(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_id': 42,
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/42')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_GET_RESULT),
        ])
        assert result['changed'] is False
        assert result['zone_id'] == 42
        assert result['zone_name'] == 'example.com'
        assert result['zone_info'] == {
            'email': 'test@example.com',
            'ttl': 10800,
            'dnssec': False,
            'dnssec_email': None,
            'ds_records': None,
        }

    def test_get_dnssec(self, mocker):
        result = self.run_module_success(mocker, hosttech_dns_zone_info, {
            'hosttech_token': 'foo',
            'zone_name': 'foo.com',
            '_ansible_remote_tmp': '/tmp/tmp',
            '_ansible_keep_remote_files': True,
        }, [
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones', without_query=True)
            .expect_query_values('query', 'foo.com')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_LIST_RESULT),
            FetchUrlCall('GET', 200)
            .expect_header('accept', 'application/json')
            .expect_header('authorization', 'Bearer foo')
            .expect_url('https://api.ns1.hosttech.eu/api/user/v1/zones/43')
            .return_header('Content-Type', 'application/json')
            .result_json(HOSTTECH_JSON_ZONE_2_GET_RESULT),
        ])
        assert result['changed'] is False
        assert result['zone_id'] == 43
        assert result['zone_name'] == 'foo.com'
        assert result['zone_info'] == {
            'email': 'test@foo.com',
            'ttl': 10800,
            'dnssec': True,
            'dnssec_email': 'test@foo.com',
            'ds_records': [
                {
                    'key_tag': 12345,
                    'algorithm': 8,
                    'digest_type': 1,
                    'digest': '012356789ABCDEF0123456789ABCDEF012345678',
                    'flags': 257,
                    'protocol': 3,
                    'public_key':
                        'MuhdzsQdqEGShwjtJDKZZjdKqUSGluFzTTinpuEeIRzLLcgkwgAPKWFa '
                        'eQntNlmcNDeCziGwpdvhJnvKXEMbFcZwsaDIJuWqERxAQNGABWfPlCLh '
                        'HQPnbpRPNKipSdBaUhuOubvFvjBpFAwiwSAapRDVsAgKvjXucfXpFfYb '
                        'pCundbAXBWhbpHVbqgmGoixXzFSwUsGVYLPpBCiDlLJwzjRKYYaoVYge '
                        'kMtKFYUVnWIKbectWkDFdVqXwkKigCUDiuTTJxOBRJRNzGiDNMWBjYSm '
                        'bBCAHMaMYaghLbYTwyKXltdHTHwBwtswGNfpnEdSpKFzZJonBZArQfHD '
                        'lfceKgmKwEF=',
                },
                {
                    'key_tag': 12345,
                    'algorithm': 8,
                    'digest_type': 2,
                    'digest': '0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF',
                    'flags': 257,
                    'protocol': 3,
                    'public_key':
                        'MuhdzsQdqEGShwjtJDKZZjdKqUSGluFzTTinpuEeIRzLLcgkwgAPKWFa '
                        'eQntNlmcNDeCziGwpdvhJnvKXEMbFcZwsaDIJuWqERxAQNGABWfPlCLh '
                        'HQPnbpRPNKipSdBaUhuOubvFvjBpFAwiwSAapRDVsAgKvjXucfXpFfYb '
                        'pCundbAXBWhbpHVbqgmGoixXzFSwUsGVYLPpBCiDlLJwzjRKYYaoVYge '
                        'kMtKFYUVnWIKbectWkDFdVqXwkKigCUDiuTTJxOBRJRNzGiDNMWBjYSm '
                        'bBCAHMaMYaghLbYTwyKXltdHTHwBwtswGNfpnEdSpKFzZJonBZArQfHD '
                        'lfceKgmKwEF=',
                }
            ],
        }



from __future__ import absolute_import, division, print_function
__metaclass__ = type


from mock import MagicMock


from ansible_collections.community.crypto.plugins.module_utils.acme.io import (
    read_file,
    write_file,
)


TEST_TEXT = r"""1234
5678"""


def test_read_file(tmpdir):
    fn = tmpdir / 'test.txt'
    fn.write(TEST_TEXT)
    assert read_file(str(fn), 't') == TEST_TEXT
    assert read_file(str(fn), 'b') == TEST_TEXT.encode('utf-8')


def test_write_file(tmpdir):
    fn = tmpdir / 'test.txt'
    module = MagicMock()
    write_file(module, str(fn), TEST_TEXT.encode('utf-8'))
    assert fn.read() == TEST_TEXT



from __future__ import absolute_import, division, print_function
__metaclass__ = type


import pytest

from mock import MagicMock


from ansible_collections.community.crypto.plugins.module_utils.acme.errors import (
    format_error_problem,
    ACMEProtocolException,
)


TEST_FORMAT_ERROR_PROBLEM = [
    (
        {
            'type': 'foo',
        },
        '',
        'Error foo'
    ),
    (
        {
            'type': 'foo',
            'title': 'bar'
        },
        '',
        'Error "bar" (foo)'
    ),
    (
        {
            'type': 'foo',
            'detail': 'bar baz'
        },
        '',
        'Error foo: "bar baz"'
    ),
    (
        {
            'type': 'foo',
            'subproblems': []
        },
        '',
        'Error foo Subproblems:'
    ),
    (
        {
            'type': 'foo',
            'subproblems': [
                {
                    'type': 'bar',
                },
            ]
        },
        '',
        'Error foo Subproblems:\n(0) Error bar'
    ),
    (
        {
            'type': 'foo',
            'subproblems': [
                {
                    'type': 'bar',
                    'subproblems': [
                        {
                            'type': 'baz',
                        },
                    ]
                },
            ]
        },
        '',
        'Error foo Subproblems:\n(0) Error bar Subproblems:\n(0.0) Error baz'
    ),
    (
        {
            'type': 'foo',
            'title': 'Foo Error',
            'detail': 'Foo went wrong',
            'subproblems': [
                {
                    'type': 'bar',
                    'detail': 'Bar went wrong',
                    'subproblems': [
                        {
                            'type': 'baz',
                            'title': 'Baz Error',
                        },
                    ]
                },
                {
                    'type': 'bar2',
                    'title': 'Bar 2 Error',
                    'detail': 'Bar really went wrong'
                },
            ]
        },
        'X.',
        'Error "Foo Error" (foo): "Foo went wrong" Subproblems:\n'
        '(X.0) Error bar: "Bar went wrong" Subproblems:\n'
        '(X.0.0) Error "Baz Error" (baz)\n'
        '(X.1) Error "Bar 2 Error" (bar2): "Bar really went wrong"'
    ),
]


@pytest.mark.parametrize("problem, subproblem_prefix, result", TEST_FORMAT_ERROR_PROBLEM)
def test_format_error_problem(problem, subproblem_prefix, result):
    res = format_error_problem(problem, subproblem_prefix)
    assert res == result


def create_regular_response(response_text):
    response = MagicMock()
    response.read = MagicMock(return_value=response_text.encode('utf-8'))
    response.closed = False
    return response


def create_error_response():
    response = MagicMock()
    response.read = MagicMock(side_effect=AttributeError('read'))
    response.closed = True
    return response


def create_decode_error(msg):
    def f(content):
        raise Exception(msg)

    return f


TEST_ACME_PROTOCOL_EXCEPTION = [
    (
        {},
        None,
        'ACME request failed.',
        {
        },
    ),
    (
        {
            'msg': 'Foo',
            'extras': {
                'foo': 'bar',
            },
        },
        None,
        'Foo.',
        {
            'foo': 'bar',
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
        },
        None,
        'ACME request failed for https://ca.example.com/foo with HTTP status 201.',
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'response': create_regular_response('xxx'),
        },
        None,
        'ACME request failed for https://ca.example.com/foo with HTTP status 201. The raw error result: xxx',
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'response': create_regular_response('xxx'),
        },
        create_decode_error('yyy'),
        'ACME request failed for https://ca.example.com/foo with HTTP status 201. The raw error result: xxx',
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'response': create_regular_response('xxx'),
        },
        lambda content: dict(foo='bar'),
        "ACME request failed for https://ca.example.com/foo with HTTP status 201. The JSON error result: {'foo': 'bar'}",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'response': create_error_response(),
        },
        None,
        'ACME request failed for https://ca.example.com/foo with HTTP status 201.',
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
                'body': 'xxx',
            },
            'response': create_error_response(),
        },
        lambda content: dict(foo='bar'),
        "ACME request failed for https://ca.example.com/foo with HTTP status 201. The JSON error result: {'foo': 'bar'}",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'content': 'xxx',
        },
        None,
        "ACME request failed for https://ca.example.com/foo with HTTP status 201. The raw error result: xxx",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 400,
            },
            'content_json': {
                'foo': 'bar',
            },
            'extras': {
                'bar': 'baz',
            }
        },
        None,
        "ACME request failed for https://ca.example.com/foo with HTTP status 400. The JSON error result: {'foo': 'bar'}",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 400,
            'bar': 'baz',
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 201,
            },
            'content_json': {
                'type': 'foo',
            },
        },
        None,
        "ACME request failed for https://ca.example.com/foo with HTTP status 201. The JSON error result: {'type': 'foo'}",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 201,
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 400,
            },
            'content_json': {
                'type': 'foo',
            },
        },
        None,
        "ACME request failed for https://ca.example.com/foo with status 400. Error foo.",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 400,
            'problem': {
                'type': 'foo',
            },
            'subproblems': [],
        },
    ),
    (
        {
            'info': {
                'url': 'https://ca.example.com/foo',
                'status': 400,
            },
            'content_json': {
                'type': 'foo',
                'title': 'Foo Error',
                'subproblems': [
                    {
                        'type': 'bar',
                        'detail': 'This is a bar error',
                        'details': 'Details.',
                    },
                ],
            },
        },
        None,
        "ACME request failed for https://ca.example.com/foo with status 400. Error \"Foo Error\" (foo). Subproblems:\n"
        "(0) Error bar: \"This is a bar error\".",
        {
            'http_url': 'https://ca.example.com/foo',
            'http_status': 400,
            'problem': {
                'type': 'foo',
                'title': 'Foo Error',
            },
            'subproblems': [
                {
                    'type': 'bar',
                    'detail': 'This is a bar error',
                    'details': 'Details.',
                },
            ],
        },
    ),
]


@pytest.mark.parametrize("input, from_json, msg, args", TEST_ACME_PROTOCOL_EXCEPTION)
def test_acme_protocol_exception(input, from_json, msg, args):
    if from_json is None:
        module = None
    else:
        module = MagicMock()
        module.from_json = from_json
    with pytest.raises(ACMEProtocolException) as exc:
        raise ACMEProtocolException(module, **input)

    print(exc.value.msg)
    print(exc.value.module_fail_args)
    print(msg)
    print(args)
    assert exc.value.msg == msg
    assert exc.value.module_fail_args == args



# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible_collections.arista.eos.tests.unit.compat.mock import patch
from ansible_collections.arista.eos.plugins.modules import eos_user
from ansible_collections.arista.eos.tests.unit.modules.utils import (
    set_module_args,
)
from .eos_module import TestEosModule, load_fixture


class TestEosUserModule(TestEosModule):

    module = eos_user

    def setUp(self):
        super(TestEosUserModule, self).setUp()

        self.mock_get_config = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_user.get_config"
        )
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_user.load_config"
        )
        self.load_config = self.mock_load_config.start()

        self.mock_get_os_version = patch(
            "ansible_collections.arista.eos.plugins.modules.eos_user.get_os_version"
        )
        self.get_os_version = self.mock_get_os_version.start()
        self.get_os_version.return_value = (4, 20, 10)

    def tearDown(self):
        super(TestEosUserModule, self).tearDown()

        self.mock_get_config.stop()
        self.mock_load_config.stop()
        self.mock_get_os_version.stop()

    def load_fixtures(self, commands=None, transport="cli"):
        self.get_config.return_value = load_fixture("eos_user_config.cfg")
        self.load_config.return_value = dict(diff=None, session="session")

    def test_eos_user_create(self):
        set_module_args(dict(name="test", nopassword=True))
        commands = ["username test nopassword"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_delete(self):
        set_module_args(dict(name="ansible", state="absent"))
        commands = ["no username ansible"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_password(self):
        set_module_args(dict(name="ansible", configured_password="test"))
        commands = ["username ansible secret test"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_privilege(self):
        set_module_args(
            dict(name="ansible", privilege=15, configured_password="test")
        )
        result = self.execute_module(changed=True)
        self.assertIn("username ansible privilege 15", result["commands"])

    def test_eos_user_privilege_invalid(self):
        set_module_args(
            dict(name="ansible", privilege=25, configured_password="test")
        )
        self.execute_module(failed=True)

    def test_eos_user_purge(self):
        set_module_args(dict(purge=True))
        commands = ["no username ansible"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_role(self):
        set_module_args(
            dict(name="ansible", role="test", configured_password="test")
        )
        result = self.execute_module(changed=True)
        self.assertIn("username ansible role test", result["commands"])

    def test_eos_user_sshkey(self):
        set_module_args(dict(name="ansible", sshkey="test"))
        commands = ["username ansible sshkey test"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_sshkey_4_23(self):
        self.get_os_version.return_value = (4, 23, 00)
        set_module_args(dict(name="ansible", sshkey="test"))
        commands = ["username ansible ssh-key test"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_update_password_changed(self):
        set_module_args(
            dict(
                name="test",
                configured_password="test",
                update_password="on_create",
            )
        )
        commands = ["username test secret test"]
        self.execute_module(changed=True, commands=commands)

    def test_eos_user_update_password_on_create_ok(self):
        set_module_args(
            dict(
                name="ansible",
                configured_password="test",
                update_password="on_create",
            )
        )
        self.execute_module()

    def test_eos_user_update_password_always(self):
        set_module_args(
            dict(
                name="ansible",
                configured_password="test",
                update_password="always",
            )
        )
        commands = ["username ansible secret test"]
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.ise.plugins.plugin_utils.ise import (
    ISESDK,
    ise_argument_spec,
    ise_compare_equality,
    get_dict_result,
)
from ansible_collections.cisco.ise.plugins.plugin_utils.exceptions import (
    InconsistentParameters,
)

# Get common arguments specification
argument_spec = ise_argument_spec()
# Add arguments specific for this module
argument_spec.update(dict(
    state=dict(type="str", default="present", choices=["present"]),
    id=dict(type="str"),
    enableAci=dict(type="bool"),
    ipAddressHostName=dict(type="str"),
    adminName=dict(type="str"),
    adminPassword=dict(type="str"),
    aciipaddress=dict(type="str"),
    aciuserName=dict(type="str"),
    acipassword=dict(type="str"),
    tenantName=dict(type="str"),
    l3RouteNetwork=dict(type="str"),
    suffixToEpg=dict(type="str"),
    suffixToSgt=dict(type="str"),
    allSxpDomain=dict(type="bool"),
    specificSxpDomain=dict(type="bool"),
    specifixSxpDomainList=dict(type="list"),
    enableDataPlane=dict(type="bool"),
    untaggedPacketIepgName=dict(type="str"),
    defaultSgtName=dict(type="str"),
    enableElementsLimit=dict(type="bool"),
    maxNumIepgFromAci=dict(type="int"),
    maxNumSgtToAci=dict(type="int"),
    aci50=dict(type="bool"),
    aci51=dict(type="bool"),
))

required_if = [
    ("state", "present", ["id"], True),
]
required_one_of = []
mutually_exclusive = []
required_together = []


class AciSettings(object):
    def __init__(self, params, ise):
        self.ise = ise
        self.new_object = dict(
            id=params.get("id"),
            enable_aci=params.get("enableAci"),
            ip_address_host_name=params.get("ipAddressHostName"),
            admin_name=params.get("adminName"),
            admin_password=params.get("adminPassword"),
            aciipaddress=params.get("aciipaddress"),
            aciuser_name=params.get("aciuserName"),
            acipassword=params.get("acipassword"),
            tenant_name=params.get("tenantName"),
            l3_route_network=params.get("l3RouteNetwork"),
            suffix_to_epg=params.get("suffixToEpg"),
            suffix_to_sgt=params.get("suffixToSgt"),
            all_sxp_domain=params.get("allSxpDomain"),
            specific_sxp_domain=params.get("specificSxpDomain"),
            specifix_sxp_domain_list=params.get("specifixSxpDomainList"),
            enable_data_plane=params.get("enableDataPlane"),
            untagged_packet_iepg_name=params.get("untaggedPacketIepgName"),
            default_sgt_name=params.get("defaultSgtName"),
            enable_elements_limit=params.get("enableElementsLimit"),
            max_num_iepg_from_aci=params.get("maxNumIepgFromAci"),
            max_num_sgt_to_aci=params.get("maxNumSgtToAci"),
            aci50=params.get("aci50"),
            aci51=params.get("aci51"),
        )

    def get_object_by_name(self, name):
        # NOTICE: Does not have a get by name method or it is in another action
        result = None
        items = self.ise.exec(
            family="aci_settings",
            function="get_aci_settings"
        ).response['AciSettings']
        result = get_dict_result(items, 'name', name)
        return result

    def get_object_by_id(self, id):
        # NOTICE: Does not have a get by id method or it is in another action
        try:
            result = self.ise.exec(
                family="aci_settings",
                function="get_aci_settings",
                handle_func_exception=False,
            ).response['AciSettings']
            # result = get_dict_result(result, 'id', id)
        except Exception as e:
            result = None
        return result

    def exists(self):
        prev_obj = None
        id_exists = False
        name_exists = False
        o_id = self.new_object.get("id")
        name = self.new_object.get("name")
        if o_id:
            prev_obj = self.get_object_by_id(o_id)
            id_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if not id_exists and name:
            prev_obj = self.get_object_by_name(name)
            name_exists = prev_obj is not None and isinstance(prev_obj, dict)
        if name_exists:
            _id = prev_obj.get("id")
            if id_exists and name_exists and o_id != _id:
                raise InconsistentParameters("The 'id' and 'name' params don't refer to the same object")
        it_exists = prev_obj is not None and isinstance(prev_obj, dict)
        return (it_exists, prev_obj)

    def requires_update(self, current_obj):
        requested_obj = self.new_object

        obj_params = [
            ("id", "id"),
            ("enableAci", "enable_aci"),
            ("ipAddressHostName", "ip_address_host_name"),
            ("adminName", "admin_name"),
            ("adminPassword", "admin_password"),
            ("aciipaddress", "aciipaddress"),
            ("aciuserName", "aciuser_name"),
            ("acipassword", "acipassword"),
            ("tenantName", "tenant_name"),
            ("l3RouteNetwork", "l3_route_network"),
            ("suffixToEpg", "suffix_to_epg"),
            ("suffixToSgt", "suffix_to_sgt"),
            ("allSxpDomain", "all_sxp_domain"),
            ("specificSxpDomain", "specific_sxp_domain"),
            ("specifixSxpDomainList", "specifix_sxp_domain_list"),
            ("enableDataPlane", "enable_data_plane"),
            ("untaggedPacketIepgName", "untagged_packet_iepg_name"),
            ("defaultSgtName", "default_sgt_name"),
            ("enableElementsLimit", "enable_elements_limit"),
            ("maxNumIepgFromAci", "max_num_iepg_from_aci"),
            ("maxNumSgtToAci", "max_num_sgt_to_aci"),
            ("aci50", "aci50"),
            ("aci51", "aci51"),
        ]
        # Method 1. Params present in request (Ansible) obj are the same as the current (ISE) params
        # If any does not have eq params, it requires update
        return any(not ise_compare_equality(current_obj.get(ise_param),
                                            requested_obj.get(ansible_param))
                   for (ise_param, ansible_param) in obj_params)

    def update(self):
        id = self.new_object.get("id")
        name = self.new_object.get("name")
        result = None
        if not id:
            id_ = self.get_object_by_name(name).get("id")
            self.new_object.update(dict(id=id_))
        result = self.ise.exec(
            family="aci_settings",
            function="update_aci_settings_by_id",
            params=self.new_object
        ).response
        return result


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail("ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'")
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        ise = ISESDK(params=self._task.args)
        obj = AciSettings(self._task.args, ise)

        state = self._task.args.get("state")

        response = None
        if state == "present":
            (obj_exists, prev_obj) = obj.exists()
            if obj_exists:
                if obj.requires_update(prev_obj):
                    ise_update_response = obj.update()
                    self._result.update(dict(ise_update_response=ise_update_response))
                    (obj_exists, updated_obj) = obj.exists()
                    response = updated_obj
                    ise.object_updated()
                else:
                    response = prev_obj
                    ise.object_already_present()
            else:
                ise.fail_json("Object does not exists, plugin only has update")

        self._result.update(dict(ise_response=response))
        self._result.update(ise.exit_json())
        return self._result



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: aws_api_gateway_domain
short_description: Manage AWS API Gateway custom domains
description:
     - Manages API Gateway custom domains for API GW Rest APIs.
     - AWS API Gateway custom domain setups use CloudFront behind the scenes.
       So you will get a CloudFront distribution as a result, configured to be aliased with your domain.
version_added: '3.3.0'
author:
    - 'Stefan Horning (@stefanhorning)'
options:
  domain_name:
    description:
      - Domain name you want to use for your API GW deployment.
    required: true
    type: str
  certificate_arn:
    description:
      - AWS Certificate Manger (ACM) TLS certificate ARN.
    type: str
    required: true
  security_policy:
    description:
      - Set allowed TLS versions through AWS defined policies. Currently only C(TLS_1_0) and C(TLS_1_2) are available.
    default: TLS_1_2
    choices: ['TLS_1_0', 'TLS_1_2']
    type: str
  endpoint_type:
    description:
      - API endpoint configuration for domain. Use EDGE for edge-optimized endpoint, or use C(REGIONAL) or C(PRIVATE).
    default: EDGE
    choices: ['EDGE', 'REGIONAL', 'PRIVATE']
    type: str
  domain_mappings:
    description:
      - Map your domain base paths to your API GW REST APIs, that you previously created. Use provided ID of the API setup and the release stage.
      - "domain_mappings should be a list of dictionaries containing three keys: base_path, rest_api_id and stage."
      - "Example: I([{ base_path: v1, rest_api_id: abc123, stage: production }])"
      - if you want base path to be just I(/) omit the param completely or set it to empty string.
    required: true
    type: list
    elements: dict
  state:
    description:
      - Create or delete custom domain setup.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
extends_documentation_fragment:
    - amazon.aws.aws
    - amazon.aws.ec2
notes:
   - Does not create a DNS entry on Route53, for that use the route53 module.
   - Only supports TLS certificates from AWS ACM that can just be referenced by the ARN, while the AWS API still offers (deprecated)
     options to add own Certificates.
'''

EXAMPLES = '''
- name: Setup endpoint for a custom domain for your API Gateway HTTP API
  community.aws.aws_api_gateway_domain:
    domain_name: myapi.foobar.com
    certificate_arn: 'arn:aws:acm:us-east-1:1231123123:certificate/8bd89412-abc123-xxxxx'
    security_policy: TLS_1_2
    endpoint_type: EDGE
    domain_mappings:
        - { rest_api_id: abc123, stage: production }
    state: present
  register: api_gw_domain_result

- name: Create a DNS record for your custom domain on route 53 (using route53 module)
  community.aws.route53:
    record: myapi.foobar.com
    value: "{{ api_gw_domain_result.response.domain.distribution_domain_name }}"
    type: A
    alias: true
    zone: foobar.com
    alias_hosted_zone_id: "{{ api_gw_domain_result.response.domain.distribution_hosted_zone_id }}"
    command: create
'''

RETURN = '''
response:
  description: The data returned by create_domain_name (or update and delete) and create_base_path_mapping methods by boto3.
  returned: success
  type: dict
  sample:
    domain:
        {
            domain_name: mydomain.com,
            certificate_arn: 'arn:aws:acm:xxxxxx',
            distribution_domain_name: xxxx.cloudfront.net,
            distribution_hosted_zone_id: ABC123123,
            endpoint_configuration: { types: ['EDGE'] },
            domain_name_status: 'AVAILABLE',
            security_policy: TLS_1_2,
            tags: {}
        }
    path_mappings: [
        { base_path: '(empty)', rest_api_id: 'abcd123', stage: 'production' }
    ]
'''

try:
    from botocore.exceptions import ClientError, BotoCoreError, EndpointConnectionError
except ImportError:
    pass  # caught by imported AnsibleAWSModule

import copy

from ansible_collections.amazon.aws.plugins.module_utils.core import AnsibleAWSModule, is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AWSRetry
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict, snake_dict_to_camel_dict


def get_domain(module, client):
    domain_name = module.params.get('domain_name')
    result = {}
    try:
        result['domain'] = get_domain_name(client, domain_name)
        result['path_mappings'] = get_domain_mappings(client, domain_name)
    except is_boto3_error_code('NotFoundException'):
        return None
    except (ClientError, BotoCoreError, EndpointConnectionError) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="getting API GW domain")
    return camel_dict_to_snake_dict(result)


def create_domain(module, client):
    path_mappings = module.params.get('domain_mappings', [])
    domain_name = module.params.get('domain_name')
    result = {'domain': {}, 'path_mappings': []}

    try:
        result['domain'] = create_domain_name(
            module,
            client,
            domain_name,
            module.params.get('certificate_arn'),
            module.params.get('endpoint_type'),
            module.params.get('security_policy')
        )

        for mapping in path_mappings:
            base_path = mapping.get('base_path', '')
            rest_api_id = mapping.get('rest_api_id')
            stage = mapping.get('stage')
            if rest_api_id is None or stage is None:
                module.fail_json('Every domain mapping needs a rest_api_id and stage name')

            result['path_mappings'].append(add_domain_mapping(client, domain_name, base_path, rest_api_id, stage))

    except (ClientError, BotoCoreError, EndpointConnectionError) as e:
        module.fail_json_aws(e, msg="creating API GW domain")
    return camel_dict_to_snake_dict(result)


def update_domain(module, client, existing_domain):
    domain_name = module.params.get('domain_name')
    result = existing_domain
    result['updated'] = False

    domain = existing_domain.get('domain')
    # Compare only relevant set of domain arguments.
    # As get_domain_name gathers all kind of state information that can't be set anyways.
    # Also this module doesn't support custom TLS cert setup params as they are kind of deprecated already and would increase complexity.
    existing_domain_settings = {
        'certificate_arn': domain.get('certificate_arn'),
        'security_policy': domain.get('security_policy'),
        'endpoint_type': domain.get('endpoint_configuration').get('types')[0]
    }
    specified_domain_settings = {
        'certificate_arn': module.params.get('certificate_arn'),
        'security_policy': module.params.get('security_policy'),
        'endpoint_type': module.params.get('endpoint_type')
    }

    if specified_domain_settings != existing_domain_settings:
        try:
            result['domain'] = update_domain_name(client, domain_name, **snake_dict_to_camel_dict(specified_domain_settings))
            result['updated'] = True
        except (ClientError, BotoCoreError, EndpointConnectionError) as e:
            module.fail_json_aws(e, msg="updating API GW domain")

    existing_mappings = copy.deepcopy(existing_domain.get('path_mappings', []))
    # Cleanout `base_path: "(none)"` elements from dicts as those won't match with specified mappings
    for mapping in existing_mappings:
        if mapping.get('base_path', 'missing') == '(none)':
            mapping.pop('base_path')

    specified_mappings = copy.deepcopy(module.params.get('domain_mappings', []))
    # Cleanout `base_path: ""` elements from dicts as those won't match with existing mappings
    for mapping in specified_mappings:
        if mapping.get('base_path', 'missing') == '':
            mapping.pop('base_path')

    if specified_mappings != existing_mappings:
        try:
            # When lists missmatch delete all existing mappings before adding new ones as specified
            for mapping in existing_domain.get('path_mappings', []):
                delete_domain_mapping(client, domain_name, mapping['base_path'])
            for mapping in module.params.get('domain_mappings', []):
                result['path_mappings'] = add_domain_mapping(
                    client, domain_name, mapping.get('base_path', ''), mapping.get('rest_api_id'), mapping.get('stage')
                )
                result['updated'] = True
        except (ClientError, BotoCoreError, EndpointConnectionError) as e:
            module.fail_json_aws(e, msg="updating API GW domain mapping")

    return camel_dict_to_snake_dict(result)


def delete_domain(module, client):
    domain_name = module.params.get('domain_name')
    try:
        result = delete_domain_name(client, domain_name)
    except (ClientError, BotoCoreError, EndpointConnectionError) as e:
        module.fail_json_aws(e, msg="deleting API GW domain")
    return camel_dict_to_snake_dict(result)


retry_params = {"tries": 10, "delay": 5, "backoff": 1.2}


@AWSRetry.backoff(**retry_params)
def get_domain_name(client, domain_name):
    return client.get_domain_name(domainName=domain_name)


@AWSRetry.backoff(**retry_params)
def get_domain_mappings(client, domain_name):
    return client.get_base_path_mappings(domainName=domain_name, limit=200).get('items', [])


@AWSRetry.backoff(**retry_params)
def create_domain_name(module, client, domain_name, certificate_arn, endpoint_type, security_policy):
    endpoint_configuration = {'types': [endpoint_type]}

    if endpoint_type == 'EDGE':
        return client.create_domain_name(
            domainName=domain_name,
            certificateArn=certificate_arn,
            endpointConfiguration=endpoint_configuration,
            securityPolicy=security_policy
        )
    else:
        # Use regionalCertificateArn for regional domain deploys
        return client.create_domain_name(
            domainName=domain_name,
            regionalCertificateArn=certificate_arn,
            endpointConfiguration=endpoint_configuration,
            securityPolicy=security_policy
        )


@AWSRetry.backoff(**retry_params)
def add_domain_mapping(client, domain_name, base_path, rest_api_id, stage):
    return client.create_base_path_mapping(domainName=domain_name, basePath=base_path, restApiId=rest_api_id, stage=stage)


@AWSRetry.backoff(**retry_params)
def update_domain_name(client, domain_name, **kwargs):
    patch_operations = []

    for key, value in kwargs.items():
        path = "/" + key
        if key == "endpointType":
            continue
        patch_operations.append({"op": "replace", "path": path, "value": value})

    return client.update_domain_name(domainName=domain_name, patchOperations=patch_operations)


@AWSRetry.backoff(**retry_params)
def delete_domain_name(client, domain_name):
    return client.delete_domain_name(domainName=domain_name)


@AWSRetry.backoff(**retry_params)
def delete_domain_mapping(client, domain_name, base_path):
    return client.delete_base_path_mapping(domainName=domain_name, basePath=base_path)


def main():
    argument_spec = dict(
        domain_name=dict(type='str', required=True),
        certificate_arn=dict(type='str', required=True),
        security_policy=dict(type='str', default='TLS_1_2', choices=['TLS_1_0', 'TLS_1_2']),
        endpoint_type=dict(type='str', default='EDGE', choices=['EDGE', 'REGIONAL', 'PRIVATE']),
        domain_mappings=dict(type='list', required=True, elements='dict'),
        state=dict(type='str', default='present', choices=['present', 'absent'])
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=False
    )

    client = module.client('apigateway')

    state = module.params.get('state')
    changed = False

    if state == "present":
        existing_domain = get_domain(module, client)
        if existing_domain is not None:
            result = update_domain(module, client, existing_domain)
            changed = result['updated']
        else:
            result = create_domain(module, client)
            changed = True
    if state == "absent":
        result = delete_domain(module, client)
        changed = True

    exit_args = {"changed": changed}

    if result is not None:
        exit_args['response'] = result

    module.exit_json(**exit_args)


if __name__ == '__main__':
    main()
